#!/usr/bin/python3

from PySide6 import QtWidgets
from ..core import AbstractWidget


class Widget(AbstractWidget, QtWidgets.QWidget):
    def __init__(self, **kwargs):
        QtWidgets.QWidget.__init__(self)
        super().__init__(**kwargs)


class Frame(AbstractWidget, QtWidgets.QFrame):
    def __init__(self, **kwargs):
        QtWidgets.QFrame.__init__(self)
        super().__init__(**kwargs)


class CalendarWidget(AbstractWidget, QtWidgets.QCalendarWidget):
    def __init__(self, **kwargs):
        QtWidgets.QCalendarWidget.__init__(self)
        super().__init__(**kwargs)


class CheckBox(AbstractWidget, QtWidgets.QCheckBox):
    def __init__(self, **kwargs):
        QtWidgets.QCheckBox.__init__(self)
        super().__init__(**kwargs)


class ColorDialog(AbstractWidget, QtWidgets.QColorDialog):
    def __init__(self, **kwargs):
        QtWidgets.QColorDialog.__init__(self)
        super().__init__(**kwargs)


class ColumnView(AbstractWidget, QtWidgets.QColumnView):
    def __init__(self, **kwargs):
        QtWidgets.QColumnView.__init__(self)
        super().__init__(**kwargs)


class ComboBox(AbstractWidget, QtWidgets.QComboBox):
    def __init__(self, **kwargs):
        QtWidgets.QComboBox.__init__(self)
        super().__init__(**kwargs)


class CommandLinkButton(AbstractWidget, QtWidgets.QCommandLinkButton):
    def __init__(self, **kwargs):
        QtWidgets.QCommandLinkButton.__init__(self)
        super().__init__(**kwargs)


class DateEdit(AbstractWidget, QtWidgets.QDateEdit):
    def __init__(self, **kwargs):
        QtWidgets.QDateEdit.__init__(self)
        super().__init__(**kwargs)


class DateTimeEdit(AbstractWidget, QtWidgets.QDateTimeEdit):
    def __init__(self, **kwargs):
        QtWidgets.QDateTimeEdit.__init__(self)
        super().__init__(**kwargs)


class Dial(AbstractWidget, QtWidgets.QDial):
    def __init__(self, **kwargs):
        QtWidgets.QDial.__init__(self)
        super().__init__(**kwargs)


class Dialog(AbstractWidget, QtWidgets.QDialog):
    def __init__(self, **kwargs):
        QtWidgets.QDialog.__init__(self)
        super().__init__(**kwargs)


class DialogButtonBox(AbstractWidget, QtWidgets.QDialogButtonBox):
    def __init__(self, **kwargs):
        QtWidgets.QDialogButtonBox.__init__(self)
        super().__init__(**kwargs)


class DockWidget(AbstractWidget, QtWidgets.QDockWidget):
    def __init__(self, **kwargs):
        QtWidgets.QDockWidget.__init__(self)
        super().__init__(**kwargs)


class DoubleSpinBox(AbstractWidget, QtWidgets.QDoubleSpinBox):
    def __init__(self, **kwargs):
        QtWidgets.QDoubleSpinBox.__init__(self)
        super().__init__(**kwargs)


class ErrorMessage(AbstractWidget, QtWidgets.QErrorMessage):
    def __init__(self, **kwargs):
        QtWidgets.QErrorMessage.__init__(self)
        super().__init__(**kwargs)


class FileDialog(AbstractWidget, QtWidgets.QFileDialog):
    def __init__(self, **kwargs):
        QtWidgets.QFileDialog.__init__(self)
        super().__init__(**kwargs)


class FocusFrame(AbstractWidget, QtWidgets.QFocusFrame):
    def __init__(self, **kwargs):
        QtWidgets.QFocusFrame.__init__(self)
        super().__init__(**kwargs)


class FontComboBox(AbstractWidget, QtWidgets.QFontComboBox):
    def __init__(self, **kwargs):
        QtWidgets.QFontComboBox.__init__(self)
        super().__init__(**kwargs)


class FontDialog(AbstractWidget, QtWidgets.QFontDialog):
    def __init__(self, **kwargs):
        QtWidgets.QFontDialog.__init__(self)
        super().__init__(**kwargs)


class GraphicsProxyWidget(AbstractWidget, QtWidgets.QGraphicsProxyWidget):
    def __init__(self, **kwargs):
        QtWidgets.QGraphicsProxyWidget.__init__(self)
        super().__init__(**kwargs)


class GraphicsView(AbstractWidget, QtWidgets.QGraphicsView):
    def __init__(self, **kwargs):
        QtWidgets.QGraphicsView.__init__(self)
        super().__init__(**kwargs)


class GraphicsWidget(AbstractWidget, QtWidgets.QGraphicsWidget):
    def __init__(self, **kwargs):
        QtWidgets.QGraphicsWidget.__init__(self)
        super().__init__(**kwargs)


class HeaderView(AbstractWidget, QtWidgets.QHeaderView):
    def __init__(self, **kwargs):
        QtWidgets.QHeaderView.__init__(self)
        super().__init__(**kwargs)


class InputDialog(AbstractWidget, QtWidgets.QInputDialog):
    def __init__(self, **kwargs):
        QtWidgets.QInputDialog.__init__(self)
        super().__init__(**kwargs)


class KeySequenceEdit(AbstractWidget, QtWidgets.QKeySequenceEdit):
    def __init__(self, **kwargs):
        QtWidgets.QKeySequenceEdit.__init__(self)
        super().__init__(**kwargs)


class LCDNumber(AbstractWidget, QtWidgets.QLCDNumber):
    def __init__(self, **kwargs):
        QtWidgets.QLCDNumber.__init__(self)
        super().__init__(**kwargs)


class LineEdit(AbstractWidget, QtWidgets.QLineEdit):
    def __init__(self, **kwargs):
        QtWidgets.QLineEdit.__init__(self)
        super().__init__(**kwargs)


class ListView(AbstractWidget, QtWidgets.QListView):
    def __init__(self, **kwargs):
        QtWidgets.QListView.__init__(self)
        super().__init__(**kwargs)


class ListWidget(AbstractWidget, QtWidgets.QListWidget):
    def __init__(self, **kwargs):
        QtWidgets.QListWidget.__init__(self)
        super().__init__(**kwargs)

    def addItemDelegate(self, delegate: QtWidgets.QAbstractItemDelegate) -> None:
        self.setItemDelegate(delegate)

    def addItemWidget(self, item: QtWidgets.QListWidgetItem, widget: QtWidgets.QWidget) -> None:
        self.addItem(item)
        self.setItemWidget(item, widget)


class MdiArea(AbstractWidget, QtWidgets.QMdiArea):
    def __init__(self, **kwargs):
        QtWidgets.QMdiArea.__init__(self)
        super().__init__(**kwargs)


class MdiSubWindow(AbstractWidget, QtWidgets.QMdiSubWindow):
    def __init__(self, **kwargs):
        QtWidgets.QMdiSubWindow.__init__(self)
        super().__init__(**kwargs)


class MenuBar(AbstractWidget, QtWidgets.QMenuBar):
    def __init__(self, **kwargs):
        QtWidgets.QMenuBar.__init__(self)
        super().__init__(**kwargs)


class MessageBox(AbstractWidget, QtWidgets.QMessageBox):
    def __init__(self, **kwargs):
        QtWidgets.QMessageBox.__init__(self)
        super().__init__(**kwargs)


class PlainTextEdit(AbstractWidget, QtWidgets.QPlainTextEdit):
    def __init__(self, **kwargs):
        QtWidgets.QPlainTextEdit.__init__(self)
        super().__init__(**kwargs)


class ProgressBar(AbstractWidget, QtWidgets.QProgressBar):
    def __init__(self, **kwargs):
        QtWidgets.QProgressBar.__init__(self)
        super().__init__(**kwargs)


class ProgressDialog(AbstractWidget, QtWidgets.QProgressDialog):
    def __init__(self, **kwargs):
        QtWidgets.QProgressDialog.__init__(self)
        super().__init__(**kwargs)


class RadioButton(AbstractWidget, QtWidgets.QRadioButton):
    def __init__(self, **kwargs):
        QtWidgets.QRadioButton.__init__(self)
        super().__init__(**kwargs)


class RubberBand(AbstractWidget, QtWidgets.QRubberBand):
    def __init__(self, **kwargs):
        QtWidgets.QRubberBand.__init__(self)
        super().__init__(**kwargs)


class ScrollBar(AbstractWidget, QtWidgets.QScrollBar):
    def __init__(self, **kwargs):
        QtWidgets.QScrollBar.__init__(self)
        super().__init__(**kwargs)


class SizeGrip(AbstractWidget, QtWidgets.QSizeGrip):
    def __init__(self, **kwargs):
        QtWidgets.QSizeGrip.__init__(self)
        super().__init__(**kwargs)


class Slider(AbstractWidget, QtWidgets.QSlider):
    def __init__(self, **kwargs):
        QtWidgets.QSlider.__init__(self)
        super().__init__(**kwargs)


class SpinBox(AbstractWidget, QtWidgets.QSpinBox):
    def __init__(self, **kwargs):
        QtWidgets.QSpinBox.__init__(self)
        super().__init__(**kwargs)


class SplashScreen(AbstractWidget, QtWidgets.QSplashScreen):
    def __init__(self, **kwargs):
        QtWidgets.QSplashScreen.__init__(self)
        super().__init__(**kwargs)


class Splitter(AbstractWidget, QtWidgets.QSplitter):
    def __init__(self, **kwargs):
        QtWidgets.QSplitter.__init__(self)
        super().__init__(**kwargs)


class SplitterHandle(AbstractWidget, QtWidgets.QSplitterHandle):
    def __init__(self, **kwargs):
        QtWidgets.QSplitterHandle.__init__(self)
        super().__init__(**kwargs)


class StatusBar(AbstractWidget, QtWidgets.QStatusBar):
    def __init__(self, **kwargs):
        QtWidgets.QStatusBar.__init__(self)
        super().__init__(**kwargs)


class TabBar(AbstractWidget, QtWidgets.QTabBar):
    def __init__(self, **kwargs):
        QtWidgets.QTabBar.__init__(self)
        super().__init__(**kwargs)


class TabWidget(AbstractWidget, QtWidgets.QTabWidget):
    def __init__(self, **kwargs):
        QtWidgets.QTabWidget.__init__(self)
        super().__init__(**kwargs)


class TableView(AbstractWidget, QtWidgets.QTableView):
    def __init__(self, **kwargs):
        QtWidgets.QTableView.__init__(self)
        super().__init__(**kwargs)


class TableWidget(AbstractWidget, QtWidgets.QTableWidget):
    def __init__(self, **kwargs):
        QtWidgets.QTableWidget.__init__(self)
        super().__init__(**kwargs)


class TextBrowser(AbstractWidget, QtWidgets.QTextBrowser):
    def __init__(self, **kwargs):
        QtWidgets.QTextBrowser.__init__(self)
        super().__init__(**kwargs)


class TextEdit(AbstractWidget, QtWidgets.QTextEdit):
    def __init__(self, **kwargs):
        QtWidgets.QTextEdit.__init__(self)
        super().__init__(**kwargs)


class TimeEdit(AbstractWidget, QtWidgets.QTimeEdit):
    def __init__(self, **kwargs):
        QtWidgets.QTimeEdit.__init__(self)
        super().__init__(**kwargs)


class ToolBar(AbstractWidget, QtWidgets.QToolBar):
    def __init__(self, **kwargs):
        QtWidgets.QToolBar.__init__(self)
        super().__init__(**kwargs)


class ToolBox(AbstractWidget, QtWidgets.QToolBox):
    def __init__(self, **kwargs):
        QtWidgets.QToolBox.__init__(self)
        super().__init__(**kwargs)


class TreeView(AbstractWidget, QtWidgets.QTreeView):
    def __init__(self, **kwargs):
        QtWidgets.QTreeView.__init__(self)
        super().__init__(**kwargs)


class TreeWidget(AbstractWidget, QtWidgets.QTreeWidget):
    def __init__(self, **kwargs):
        QtWidgets.QTreeWidget.__init__(self)
        super().__init__(**kwargs)


class UndoView(AbstractWidget, QtWidgets.QUndoView):
    def __init__(self, **kwargs):
        QtWidgets.QUndoView.__init__(self)
        super().__init__(**kwargs)


class Wizard(AbstractWidget, QtWidgets.QWizard):
    def __init__(self, **kwargs):
        QtWidgets.QWizard.__init__(self)
        super().__init__(**kwargs)


class WizardPage(AbstractWidget, QtWidgets.QWizardPage):
    def __init__(self, **kwargs):
        QtWidgets.QWizardPage.__init__(self)
        super().__init__(**kwargs)



from typing import Tuple, Union
from PySide6.QtWidgets import QFrame, QLayout, QWidget
from PySide6.QtCore import QMargins
from ..core import AbstractWidget


class FrameContainer(AbstractWidget, QFrame):
    def __init__(self, 
                 *,
                 child: Union[QWidget, QLayout] = None,
                 padding: Union[QMargins, Tuple[int, int, int, int]] = None,
                 **kwargs):
        QFrame.__init__(self)

        self.setFrameShape(QFrame.Shape.NoFrame)
        self.setFrameShadow(QFrame.Shadow.Raised)

        super().__init__(**kwargs)

        if child is not None:
            if isinstance(child, QWidget):
                child.setParent(self)
            elif isinstance(child, QLayout):
                child.setProperty("parent", self)
                self.setLayout(child)

            if padding is not None:
                self.setContentsMargins(*padding if isinstance(padding, (tuple, list)) else padding)


class WidgetContainer(AbstractWidget, QWidget):
    def __init__(self, *, child: Union[QWidget, QLayout] = None, **kwargs):
        QWidget.__init__(self)
        super().__init__(**kwargs)

        if child is not None:
            if isinstance(child, QWidget):
                child.setParent(self)
            elif isinstance(child, QLayout):
                child.setProperty("parent", self)
                self.setLayout(child)


class Container(FrameContainer):
    ...


#!/usr/bin/python3


from enum import Enum
from typing import Any, Union
from ..enums.env_vars import EnvVars
import os


class EnvVar:
    '''
    ### Set Key, and Value
    - e.g, EnvVar.set(EnvVars.scale_factor, "0")
    - e.g, EnvVar.set("QT_SCALE_FACTOR", "0")

    ### Get Value, from Key
    - e.g, EnvVar.get(EnvVars.scale_factor, "0")
    - e.g, EnvVar.get("QT_SCALE_FACTOR", "0")
    - e.g, EnvVar("QT_SCALE_FACTOR", "0")
    '''

    def __call__(self, key: Union[EnvVars, str], default: Any = None) -> Any:
        return self.get(key, default)

    @classmethod
    def set(cls, key: Union[EnvVars, str], value: Any) -> None:
        os.environ[key.value 
                   if isinstance(key, EnvVars) 
                   else key] = (value.value 
                                if isinstance(value, Enum) 
                                else value)

    @classmethod
    def get(cls, key: Union[EnvVars, str], default: Any = None) -> Any:
        return os.environ.get(key, default)



from typing import Union
from PySide6.QtWidgets import QHBoxLayout, QLayoutItem, QSpacerItem, QWidget, QLayout
from ..tools.wrappers.h_layout import HLayoutWrapper, ColumnLayoutWrapper
from ..utils.alignment import Alignment
from ..core import AbstractQObject


class HLayout(AbstractQObject, QHBoxLayout):
    def __init__(self,
                 *,
                 children: list[Union[QWidget, 
                                      QLayout, 
                                      HLayoutWrapper,
                                      ColumnLayoutWrapper,
                                      Alignment]] = None,
                 **kwargs):
        QHBoxLayout.__init__(self)
        super().__init__(**kwargs)

        if not children:
            return

        for child in children:
            if isinstance(child, Alignment):
                _widget = child.child

                if isinstance(_widget, QWidget):
                    _func = self.addWidget
                elif isinstance(_widget, QLayoutItem):
                    _func = self.addItem

                _func(_widget)
                self.setAlignment(_widget, child.alignment)

            elif isinstance(child, HLayoutWrapper):
                _widget = child.child

                if isinstance(_widget, QWidget):
                    _func = self.addWidget
                elif isinstance(_widget, QLayout):
                    _func = self.addLayout

                _func(*child._yield_attr())

            elif isinstance(child, QSpacerItem):
                self.addSpacerItem(child)

            elif isinstance(child, QLayoutItem):
                self.addItem(child)

            elif isinstance(child, QWidget):
                self.addWidget(child)

            elif isinstance(child, QLayout):
                self.addLayout(child)


class ColumnLayout(HLayout):
    ...


#!/usr/bin/python3

from PySide6.QtWidgets import QLayoutItem, QWidget, QWidgetItem
from ...enums.position import Positions


class BorderLayoutWrapper:
    def __init__(self, 
                 child: QWidget | QLayoutItem, 
                 pos: Positions = Positions.left):

        self.pos = pos
        self.item = child

        if isinstance(child, QWidget):
            self.item = QWidgetItem(child)


from typing import Union
from PySide6.QtWidgets import QFormLayout, QWidget, QLayoutItem
from ..tools.wrappers.form_layout import FormLayoutWrapper
from ..utils.alignment import Alignment
from ..core import AbstractQObject


class FormLayout(AbstractQObject, QFormLayout):
    def __init__(self,
                 *,
                 children: list[Union[QWidget, 
                                      QLayoutItem, 
                                      FormLayoutWrapper,
                                      Alignment]],
                 **kwargs):
        QFormLayout.__init__(self)
        super().__init__(**kwargs)

        if not children:
            return

        for child in children:
            if isinstance(child, Alignment):
                _widget = child.child

                if isinstance(_widget, QWidget):
                    _func = self.addWidget
                elif isinstance(_widget, QLayoutItem):
                    _func = self.addItem

                _func(_widget)
                self.setAlignment(_widget, child.alignment)

            elif isinstance(child, FormLayoutWrapper):
                if child.row is not None:
                    self.addRow(child.row)
                else:
                    self.addRow(child.label, child.field)

            elif isinstance(child, QLayoutItem):
                self.addItem(child)

            elif isinstance(child, QWidget):
                self.addWidget(child)


from typing import Union
from enum import Enum, auto
from PySide6.QtCore import QMimeData
from PySide6.QtWidgets import QApplication
from PySide6.QtGui import QClipboard, QImage, QPixmap
from ..utils.caseconverter import camelcase


class Clipboard:
    Mode = QClipboard.Mode
    class Types(Enum):
        mime_data = auto()
        pixmap = auto()
        image = auto()
        text = auto()

    @classmethod
    def paste_data(
        cls,
        dtype: Types = Types.text,
        mode: QClipboard.Mode = QClipboard.Mode.Clipboard,
    ) -> Union[QMimeData, str, QImage, QPixmap]:

        return getattr(QApplication.clipboard(), camelcase(dtype.name))(mode)

    @classmethod
    def copy_data(
        cls,
        data: Union[QMimeData, QImage, QPixmap, str],
        dtype: Types = Types.text,
        mode: QClipboard.Mode = QClipboard.Mode.Clipboard,
    ) -> None:

        return getattr(QApplication.clipboard(), camelcase("set_"+dtype.name))(data, mode)

    @classmethod
    def copy(
        cls,
        data: Union[QMimeData, QImage, QPixmap, str],
        dtype: Types = Types.text,
        mode: QClipboard.Mode = QClipboard.Mode.Clipboard
    ) -> None:
        return cls.copy_data(data, dtype, mode)

    @classmethod
    def paste(
        cls,
        dtype: Types = Types.text,
        mode: QClipboard.Mode = QClipboard.Mode.Clipboard
    ) -> Union[QMimeData, str, QImage, QPixmap]:
        return cls.paste_data(dtype, mode)


#!/usr/bin/python3

import os
import platform
from typing import Any
from enum import Enum, auto
from ..utils import EnvVar
from ..core import AbstractQObject, AbstractConfig
from PySide6.QtCore import QSettings, QTimer


class Settings(AbstractQObject, QSettings):
    '''
    Config Class
    ----
    class _Line(AbstractConfig):
        def name(self) - str:
            return "Name"

        def group(self) -> str:
            return "User"

        def get(self) -> Callable:
            return Api.fetch("user-line").text

        def set(self) -> Callable:
            return Api.fetch("user-line").setText

        def signal(self) -> Signal:
            return Api.fetch("user-line").textChanged

        def default(self) -> Any:
            ...

        def type(self) -> object:
            ...

    Usage
    -----
    Settings(
        configs=[
            _Line()
        ]
    )

    Output Result
    -------------
    [User]\n
    Name=
    '''

    class System(Enum):
        java = auto()
        linux = auto()
        darwin = auto()
        windows = auto()

        macos = darwin
        posix = linux
        unknow = auto()

    def __init__(self, *args, configs: list[AbstractConfig] = None, **kwargs):
        QSettings.__init__(self, *args)
        super().__init__(**kwargs)

        if not self.allKeys():
            self._init_configs(configs)
        else:
            self._setup_configs(configs)

    def value(self, 
              key: str,
              default: Any = None,
              type: object | None = None,
              group: str = None
              ) -> Any:

        if group is not None:
            self.beginGroup(group)
            value = super().value(key, 
                                  *(value for value in (default, type) 
                                    if value is not None))
            self.endGroup()
            return value

        return super().value(key)

    def get(self, 
              key: str,
              default: Any = None,
              type: object | None = None,
              group: str = None
              ) -> Any:
        return self.value(key, default, type, group)

    def setValue(self, key: str, value: Any, group: str = None) -> None:
        if group is not None:
            self.beginGroup(group)
            super().setValue(key, value)
            return self.endGroup()
        return super().setValue(key, value)

    def remove(self, key: str, group: str = None) -> None:
        if group is not None:
            self.beginGroup(group)
            super().remove(key)
            return self.endGroup()
        return super().remove(key)

    def _get_default_format(self) -> QSettings.Format:
        return self.defaultFormat()

    def _set_default_path(self) -> str:
        if self.system() == Settings.System.windows:
            if self.scope() == QSettings.Scope.SystemScope:
                return EnvVar.get("FOLDERID_ProgramData")
            return EnvVar.get("FOLDERID_RoamingAppData")

        if self.scope() == QSettings.Scope.SystemScope:
            return os.path.join("etc", "xdg")
        return os.path.join(EnvVar.get("HOME"), ".config")

    def _init_configs(self, configs: list[AbstractConfig]) -> None:
        for config in configs:
            config.signal().connect(
                lambda: QTimer.singleShot(0, lambda: self._update_config(config)))

    def _setup_configs(self, configs: list[AbstractConfig]) -> None:
        for config in configs:
            _value = self.value(config.name(), 
                                config.default(),
                                config.type(),
                                config.group())

            QTimer.singleShot(0, lambda: config.set()(_value))
            config.signal().connect(
                lambda: QTimer.singleShot(0, lambda: self._update_config(config)))

    def _update_config(self, config) -> None:
        self.setValue(config.name(), 
                      config.get()(),
                      config.group())

    def current_system(self) -> System:
        return Settings.System(platform.system().strip().lower() or "unknow")

    def system(self) -> System:
        return self.current_system()


#!/usr/bin/python3

from .window import *
from .dialogs import *
from ._widgets import *

from .menu import Menu
from .label import Label
from .stack import Stack
from .scroll_area import ScrollArea
from .icon_widget import IconWidget
from .line import Line, HLine, VLine
from .buttons import PushButton, ToolButton
from .application import Application, App
from .stacked_widget import StackedWidget
from .container import Container, FrameContainer, WidgetContainer
from .size_grip import WindowSizeGrip
from .group_box import GroupBox
from .eliding_label import ElidingLabel
from .video_widget import VideoWidget


#!/usr/bin/python3

from abc import ABCMeta, abstractmethod
from typing import Any, Callable
from PySide6.QtCore import Signal


class AbstractConfig(metaclass=ABCMeta):
    @abstractmethod
    def name(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def get(self) -> Callable:
        raise NotImplementedError

    @abstractmethod
    def set(self) -> Callable:
        raise NotImplementedError

    @abstractmethod
    def signal(self) -> Signal:
        raise NotImplementedError

    def group(self) -> str:
        ...

    def default(self) -> Any:
        ...

    def type(self) -> object | None:
        ...


#!/usr/bin/python3

from PySide6 import QtGui
from ...core import AbstractQObject


class Action(AbstractQObject, QtGui.QAction):
    def __init__(self, *args, **kwargs):
        QtGui.QAction.__init__(self, *args)
        super().__init__(**kwargs)


class ActionGroup(AbstractQObject, QtGui.QActionGroup):
    def __init__(self, *args, **kwargs):
        QtGui.QActionGroup.__init__(self, *args)
        super().__init__(**kwargs)


class Clipboard(AbstractQObject, QtGui.QClipboard):
    def __init__(self, *args, **kwargs):
        QtGui.QClipboard.__init__(self, *args)
        super().__init__(**kwargs)


class DoubleValidator(AbstractQObject, QtGui.QDoubleValidator):
    def __init__(self, *args, **kwargs):
        QtGui.QDoubleValidator.__init__(self, *args)
        super().__init__(**kwargs)


class Drag(AbstractQObject, QtGui.QDrag):
    def __init__(self, *args, **kwargs):
        QtGui.QDrag.__init__(self, *args)
        super().__init__(**kwargs)


class GuiApplication(AbstractQObject, QtGui.QGuiApplication):
    def __init__(self, *args, **kwargs):
        QtGui.QGuiApplication.__init__(self, *args)
        super().__init__(**kwargs)


class InputDevice(AbstractQObject, QtGui.QInputDevice):
    def __init__(self, *args, **kwargs):
        QtGui.QInputDevice.__init__(self, *args)
        super().__init__(**kwargs)


class InputMethod(AbstractQObject, QtGui.QInputMethod):
    def __init__(self, *args, **kwargs):
        QtGui.QInputMethod.__init__(self, *args)
        super().__init__(**kwargs)


class IntValidator(AbstractQObject, QtGui.QIntValidator):
    def __init__(self, *args, **kwargs):
        QtGui.QIntValidator.__init__(self, *args)
        super().__init__(**kwargs)


class OffscreenSurface(AbstractQObject, QtGui.QOffscreenSurface):
    def __init__(self, *args, **kwargs):
        QtGui.QOffscreenSurface.__init__(self, *args)
        super().__init__(**kwargs)


class OpenGLContext(AbstractQObject, QtGui.QOpenGLContext):
    def __init__(self, *args, **kwargs):
        QtGui.QOpenGLContext.__init__(self, *args)
        super().__init__(**kwargs)


class OpenGLContextGroup(AbstractQObject, QtGui.QOpenGLContextGroup):
    def __init__(self, *args, **kwargs):
        QtGui.QOpenGLContextGroup.__init__(self, *args)
        super().__init__(**kwargs)


class PaintDeviceWindow(AbstractQObject, QtGui.QPaintDeviceWindow):
    def __init__(self, *args, **kwargs):
        QtGui.QPaintDeviceWindow.__init__(self, *args)
        super().__init__(**kwargs)


class PdfWriter(AbstractQObject, QtGui.QPdfWriter):
    def __init__(self, *args, **kwargs):
        QtGui.QPdfWriter.__init__(self, *args)
        super().__init__(**kwargs)


class PointingDevice(AbstractQObject, QtGui.QPointingDevice):
    def __init__(self, *args, **kwargs):
        QtGui.QPointingDevice.__init__(self, *args)
        super().__init__(**kwargs)


class PyTextObject(AbstractQObject, QtGui.QPyTextObject):
    def __init__(self, *args, **kwargs):
        QtGui.QPyTextObject.__init__(self, *args)
        super().__init__(**kwargs)


class RasterWindow(AbstractQObject, QtGui.QRasterWindow):
    def __init__(self, *args, **kwargs):
        QtGui.QRasterWindow.__init__(self, *args)
        super().__init__(**kwargs)


class RegularExpressionValidator(AbstractQObject, QtGui.QRegularExpressionValidator):
    def __init__(self, *args, **kwargs):
        QtGui.QRegularExpressionValidator.__init__(self, *args)
        super().__init__(**kwargs)


class Screen(AbstractQObject, QtGui.QScreen):
    def __init__(self, *args, **kwargs):
        QtGui.QScreen.__init__(self, *args)
        super().__init__(**kwargs)


class SessionManager(AbstractQObject, QtGui.QSessionManager):
    def __init__(self, *args, **kwargs):
        QtGui.QSessionManager.__init__(self, *args)
        super().__init__(**kwargs)


class Shortcut(AbstractQObject, QtGui.QShortcut):
    def __init__(self, *args, **kwargs):
        QtGui.QShortcut.__init__(self, *args)
        super().__init__(**kwargs)


class StandardItemModel(AbstractQObject, QtGui.QStandardItemModel):
    def __init__(self, *args, **kwargs):
        QtGui.QStandardItemModel.__init__(self, *args)
        super().__init__(**kwargs)


class StyleHints(AbstractQObject, QtGui.QStyleHints):
    def __init__(self, *args, **kwargs):
        QtGui.QStyleHints.__init__(self, *args)
        super().__init__(**kwargs)


class SyntaxHighlighter(AbstractQObject, QtGui.QSyntaxHighlighter):
    def __init__(self, *args, **kwargs):
        QtGui.QSyntaxHighlighter.__init__(self, *args)
        super().__init__(**kwargs)


class TextBlockGroup(AbstractQObject, QtGui.QTextBlockGroup):
    def __init__(self, *args, **kwargs):
        QtGui.QTextBlockGroup.__init__(self, *args)
        super().__init__(**kwargs)


class TextDocument(AbstractQObject, QtGui.QTextDocument):
    def __init__(self, *args, **kwargs):
        QtGui.QTextDocument.__init__(self, *args)
        super().__init__(**kwargs)


class TextFrame(AbstractQObject, QtGui.QTextFrame):
    def __init__(self, *args, **kwargs):
        QtGui.QTextFrame.__init__(self, *args)
        super().__init__(**kwargs)


class TextList(AbstractQObject, QtGui.QTextList):
    def __init__(self, *args, **kwargs):
        QtGui.QTextList.__init__(self, *args)
        super().__init__(**kwargs)


class TextObject(AbstractQObject, QtGui.QTextObject):
    def __init__(self, *args, **kwargs):
        QtGui.QTextObject.__init__(self, *args)
        super().__init__(**kwargs)


class TextTable(AbstractQObject, QtGui.QTextTable):
    def __init__(self, *args, **kwargs):
        QtGui.QTextTable.__init__(self, *args)
        super().__init__(**kwargs)


class UndoGroup(AbstractQObject, QtGui.QUndoGroup):
    def __init__(self, *args, **kwargs):
        QtGui.QUndoGroup.__init__(self, *args)
        super().__init__(**kwargs)


class UndoStack(AbstractQObject, QtGui.QUndoStack):
    def __init__(self, *args, **kwargs):
        QtGui.QUndoStack.__init__(self, *args)
        super().__init__(**kwargs)


class Validator(AbstractQObject, QtGui.QValidator):
    def __init__(self, *args, **kwargs):
        QtGui.QValidator.__init__(self, *args)
        super().__init__(**kwargs)


class Window(AbstractQObject, QtGui.QWindow):
    def __init__(self, *args, **kwargs):
        QtGui.QWindow.__init__(self, *args)
        super().__init__(**kwargs)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from PySide6.QtMultimediaWidgets import QVideoWidget
from PySide6.QtMultimedia import QMediaPlayer
from ..core import AbstractWidget


class VideoWidget(AbstractWidget, QVideoWidget):
    def __init__(self, 
                 *, 
                 player: QMediaPlayer = None,
                 running: bool = False, 
                 **kwargs):
        QVideoWidget.__init__(self)
        super().__init__(**kwargs)

        if player is not None:
            player.setParent(self)
            player.setVideoOutput(self)
            if running and not player.isPlaying():
                player.play()



#!/usr/bin/python3

from PySide6 import QtCore
from ...core import AbstractTool


class BitArray(AbstractTool, QtCore.QBitArray):
    def __init__(self, *args, **kwargs):
        QtCore.QBitArray.__init__(self, *args)
        super().__init__(**kwargs)


class ByteArray(AbstractTool, QtCore.QByteArray):
    def __init__(self, *args, **kwargs):
        QtCore.QByteArray.__init__(self, *args)
        super().__init__(**kwargs)


class ByteArrayMatcher(AbstractTool, QtCore.QByteArrayMatcher):
    def __init__(self, *args, **kwargs):
        QtCore.QByteArrayMatcher.__init__(self, *args)
        super().__init__(**kwargs)


class Collator(AbstractTool, QtCore.QCollator):
    def __init__(self, *args, **kwargs):
        QtCore.QCollator.__init__(self, *args)
        super().__init__(**kwargs)


class DataStream(AbstractTool, QtCore.QDataStream):
    def __init__(self, *args, **kwargs):
        QtCore.QDataStream.__init__(self, *args)
        super().__init__(**kwargs)


class Date(AbstractTool, QtCore.QDate):
    def __init__(self, *args, **kwargs):
        QtCore.QDate.__init__(self, *args)
        super().__init__(**kwargs)


class DateTime(AbstractTool, QtCore.QDateTime):
    def __init__(self, *args, **kwargs):
        QtCore.QDateTime.__init__(self, *args)
        super().__init__(**kwargs)


class DeadlineTimer(AbstractTool, QtCore.QDeadlineTimer):
    def __init__(self, *args, **kwargs):
        QtCore.QDeadlineTimer.__init__(self, *args)
        super().__init__(**kwargs)


class Dir(AbstractTool, QtCore.QDir):
    def __init__(self, *args, **kwargs):
        QtCore.QDir.__init__(self, *args)
        super().__init__(**kwargs)


class EasingCurve(AbstractTool, QtCore.QEasingCurve):
    def __init__(self, *args, **kwargs):
        QtCore.QEasingCurve.__init__(self, *args)
        super().__init__(**kwargs)


class FileInfo(AbstractTool, QtCore.QFileInfo):
    def __init__(self, *args, **kwargs):
        QtCore.QFileInfo.__init__(self, *args)
        super().__init__(**kwargs)


class FutureInterfaceBase(AbstractTool, QtCore.QFutureInterfaceBase):
    def __init__(self, *args, **kwargs):
        QtCore.QFutureInterfaceBase.__init__(self, *args)
        super().__init__(**kwargs)


class JsonDocument(AbstractTool, QtCore.QJsonDocument):
    def __init__(self, *args, **kwargs):
        QtCore.QJsonDocument.__init__(self, *args)
        super().__init__(**kwargs)


class Line(AbstractTool, QtCore.QLine):
    def __init__(self, *args, **kwargs):
        QtCore.QLine.__init__(self, *args)
        super().__init__(**kwargs)


class LineF(AbstractTool, QtCore.QLineF):
    def __init__(self, *args, **kwargs):
        QtCore.QLineF.__init__(self, *args)
        super().__init__(**kwargs)


class Locale(AbstractTool, QtCore.QLocale):
    def __init__(self, *args, **kwargs):
        QtCore.QLocale.__init__(self, *args)
        super().__init__(**kwargs)


class LockFile(AbstractTool, QtCore.QLockFile):
    def __init__(self, *args, **kwargs):
        QtCore.QLockFile.__init__(self, *args)
        super().__init__(**kwargs)


class LoggingCategory(AbstractTool, QtCore.QLoggingCategory):
    def __init__(self, *args, **kwargs):
        QtCore.QLoggingCategory.__init__(self, *args)
        super().__init__(**kwargs)


class Margins(AbstractTool, QtCore.QMargins):
    def __init__(self, *args, **kwargs):
        QtCore.QMargins.__init__(self, *args)
        super().__init__(**kwargs)


class MarginsF(AbstractTool, QtCore.QMarginsF):
    def __init__(self, *args, **kwargs):
        QtCore.QMarginsF.__init__(self, *args)
        super().__init__(**kwargs)

if QtCore.QSysInfo.buildCpuArchitecture() != "arm64":
    class NativeIpcKey(AbstractTool, QtCore.QNativeIpcKey):
        def __init__(self, *args, **kwargs):
            QtCore.QNativeIpcKey.__init__(self, *args)
            super().__init__(**kwargs)


class Point(AbstractTool, QtCore.QPoint):
    def __init__(self, *args, **kwargs):
        QtCore.QPoint.__init__(self, *args)
        super().__init__(**kwargs)


class PointF(AbstractTool, QtCore.QPointF):
    def __init__(self, *args, **kwargs):
        QtCore.QPointF.__init__(self, *args)
        super().__init__(**kwargs)


class Rect(AbstractTool, QtCore.QRect):
    def __init__(self, *args, **kwargs):
        QtCore.QRect.__init__(self, *args)
        super().__init__(**kwargs)


class RectF(AbstractTool, QtCore.QRectF):
    def __init__(self, *args, **kwargs):
        QtCore.QRectF.__init__(self, *args)
        super().__init__(**kwargs)


class RegularExpression(AbstractTool, QtCore.QRegularExpression):
    def __init__(self, *args, **kwargs):
        QtCore.QRegularExpression.__init__(self, *args)
        super().__init__(**kwargs)


class Size(AbstractTool, QtCore.QSize):
    def __init__(self, *args, **kwargs):
        QtCore.QSize.__init__(self, *args)
        super().__init__(**kwargs)


class SizeF(AbstractTool, QtCore.QSizeF):
    def __init__(self, *args, **kwargs):
        QtCore.QSizeF.__init__(self, *args)
        super().__init__(**kwargs)


class SystemSemaphore(AbstractTool, QtCore.QSystemSemaphore):
    def __init__(self, *args, **kwargs):
        QtCore.QSystemSemaphore.__init__(self, *args)
        super().__init__(**kwargs)


class TextStream(AbstractTool, QtCore.QTextStream):
    def __init__(self, *args, **kwargs):
        QtCore.QTextStream.__init__(self, *args)
        super().__init__(**kwargs)


class Time(AbstractTool, QtCore.QTime):
    def __init__(self, *args, **kwargs):
        QtCore.QTime.__init__(self, *args)
        super().__init__(**kwargs)


class Url(AbstractTool, QtCore.QUrl):
    def __init__(self, *args, **kwargs):
        QtCore.QUrl.__init__(self, *args)
        super().__init__(**kwargs)


class UrlQuery(AbstractTool, QtCore.QUrlQuery):
    def __init__(self, *args, **kwargs):
        QtCore.QUrlQuery.__init__(self, *args)
        super().__init__(**kwargs)


class XmlStreamReader(AbstractTool, QtCore.QXmlStreamReader):
    def __init__(self, *args, **kwargs):
        QtCore.QXmlStreamReader.__init__(self, *args)
        super().__init__(**kwargs)


class XmlStreamWriter(AbstractTool, QtCore.QXmlStreamWriter):
    def __init__(self, *args, **kwargs):
        QtCore.QXmlStreamWriter.__init__(self, *args)
        super().__init__(**kwargs)



from typing import Union
from PySide6.QtCore import QPoint, Qt
from PySide6.QtGui import QMouseEvent, QResizeEvent
from PySide6.QtWidgets import (
    QLayout, 
    QVBoxLayout,
    QWidget, 
    QSizePolicy
)
from ..size_grip import WindowSizeGrip
from ...core import AbstractWidget
from .base import BaseWindow


class FramelessWindow(AbstractWidget, BaseWindow):
    def __init__(self, *,
                 home: Union[QWidget, QLayout] = None,
                 title_bar: QWidget = None,
                 size_grip: WindowSizeGrip = None,
                 **kwargs):
        BaseWindow.__init__(self, **kwargs)

        self.setWindowFlags(self.windowFlags() | Qt.WindowType.FramelessWindowHint)
        self.setAttribute(Qt.WidgetAttribute.WA_AcceptTouchEvents, True)
        self.setAttribute(Qt.WidgetAttribute.WA_Hover, True)

        self.setAutoFillBackground(True)
        self.setDocumentMode(True)

        super().__init__(**kwargs)

        self._old_pos = self.pos()

        self._is_system_move = False
        self._is_pressed = False
        self._size_grip = size_grip
        self._title_bar = title_bar

        if not title_bar:
            self.mouseMoveEvent = self._mouseMoveEvent
            self.mousePressEvent = self._mousePressEvent
            self.mouseReleaseEvent = self._mouseReleaseEvent

        if size_grip is not None:
            self._size_grip.setParent(self)

        self._set_home(home)

    def _set_home(self, home) -> None:
        if self._title_bar is not None:
            self._vlayout = QVBoxLayout()
            self._vlayout.setContentsMargins(0, 0, 0, 0)

            if isinstance(self._title_bar, QLayout):
                _widget = QWidget()
                self._title_bar.setProperty("parent", _widget)
                _widget.setLayout(self._title_bar)
                self._title_bar = _widget

            self._set_titlebar(self._title_bar)

            if home is not None:
                _func = self._vlayout.addWidget
                if isinstance(home, QLayout):
                    _func = self._vlayout.addLayout
                _func(home)

            return super()._set_home(self._vlayout)

        return super()._set_home(home)

    def _set_titlebar(self, titlebar: QWidget):
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(titlebar.sizePolicy().hasHeightForWidth())
        titlebar.setSizePolicy(sizePolicy)

        titlebar.mouseMoveEvent = self._mouseMoveEvent
        titlebar.mousePressEvent = self._mousePressEvent
        titlebar.mouseReleaseEvent = self._mouseReleaseEvent

        self._vlayout.addWidget(titlebar)

    def resizeEvent(self, event: QResizeEvent):
        if self._size_grip is not None:
            self._size_grip.updateGeometry()

        if self._title_bar is not None:
            self._title_bar.resize(self.width(), self._title_bar.height())

        return QWidget.resizeEvent(self, event)

    def _mousePressEvent(self, event: QMouseEvent) -> None:
        self._is_pressed = True
        if event.button() == event.buttons().LeftButton:
            self._old_pos = event.globalPos()
            self._is_system_move = self.windowHandle().startSystemMove()
        return super().mousePressEvent(event)

    def _mouseMoveEvent(self, event: QMouseEvent) -> None:
        if self._is_pressed:

            if self._is_system_move:
                return super().mouseMoveEvent(event)

            delta = QPoint(event.globalPos() - self._old_pos)
            self.move(self.x() + delta.x(), self.y() + delta.y())
            self._old_pos = event.globalPos()

    def _mouseReleaseEvent(self, event: QMouseEvent) -> None:
        self._is_pressed = False
        return super().mouseReleaseEvent(event)


#!/usr/bin/python3
# -*- coding: utf-8 -*-

from ..core import AbstractQObject
from PySide6.QtMultimedia import QMediaPlayer


class MediaPlayer(AbstractQObject, QMediaPlayer):
    def __init__(self, running: bool = False, **kwargs):
        QMediaPlayer.__init__(self)
        super().__init__(**kwargs)

        if (audio_out := kwargs.get("setAudioOutput", None)):
            audio_out.setParent(self)

        if (video_sink := kwargs.get("setVideoSink", None)):
            video_sink.setParent(self)

        if running:
            self.play()


# coding:utf-8
from copy import deepcopy
from typing import Callable, Union
from ._base import Func


def TryExc(func: Union[Callable, Func], *default):
    """ func for exception handling

    Parameters
    ----------
    *default:
        the default value returned when an exception occurs
    
    Example
    -------
    TryExc(lambda: func(*args, **kwargs), *default)
    """

    try:
        if isinstance(func, Func):
            return func.func()(*func.args(), **func.kwargs())
        return func()
    except BaseException:
        value = deepcopy(default)
        if len(value) == 0:
            return None
        elif len(value) == 1:
            return value[0]
        return value


from .caseconverter import CaseConverter
from .boundaries import (
    OnDelimeterUppercaseNext,
    OnUpperPrecededByLowerAppendUpper,
    OnUpperPrecededByUpperAppendCurrent,
)


class Pascal(CaseConverter):
    def init(self, input_buffer, output_buffer):
        output_buffer.write(input_buffer.read(1).upper())

    def define_boundaries(self):
        self.add_boundary_handler(OnDelimeterUppercaseNext(self.delimiters()))
        self.add_boundary_handler(OnUpperPrecededByLowerAppendUpper())
        self.add_boundary_handler(OnUpperPrecededByUpperAppendCurrent())

    def prepare_string(self, s):
        if s.isupper():
            return s.lower()

        return s

    def mutate(self, c):
        return c.lower()


def pascalcase(s, **kwargs):
    """Convert a string to pascal case

    Example

        Hello World => HelloWorld
        hello world => HelloWorld

    """
    return Pascal(s, **kwargs).convert()



#!/usr/bin/python3

from PySide6 import QtCore
from ...core import AbstractQObject


class AnimationGroup(AbstractQObject, QtCore.QAnimationGroup):
    def __init__(self, *args, **kwargs):
        QtCore.QAnimationGroup.__init__(self, *args)
        super().__init__(**kwargs)


class Buffer(AbstractQObject, QtCore.QBuffer):
    def __init__(self, *args, **kwargs):
        QtCore.QBuffer.__init__(self, *args)
        super().__init__(**kwargs)


class ConcatenateTablesProxyModel(AbstractQObject, QtCore.QConcatenateTablesProxyModel):
    def __init__(self, *args, **kwargs):
        QtCore.QConcatenateTablesProxyModel.__init__(self, *args)
        super().__init__(**kwargs)


class CoreApplication(AbstractQObject, QtCore.QCoreApplication):
    def __init__(self, *args, **kwargs):
        QtCore.QCoreApplication.__init__(self, *args)
        super().__init__(**kwargs)


class EventLoop(AbstractQObject, QtCore.QEventLoop):
    def __init__(self, *args, **kwargs):
        QtCore.QEventLoop.__init__(self, *args)
        super().__init__(**kwargs)


class FileDevice(AbstractQObject, QtCore.QFileDevice):
    def __init__(self, *args, **kwargs):
        QtCore.QFileDevice.__init__(self, *args)
        super().__init__(**kwargs)


class FileSelector(AbstractQObject, QtCore.QFileSelector):
    def __init__(self, *args, **kwargs):
        QtCore.QFileSelector.__init__(self, *args)
        super().__init__(**kwargs)


class FileSystemWatcher(AbstractQObject, QtCore.QFileSystemWatcher):
    def __init__(self, *args, **kwargs):
        QtCore.QFileSystemWatcher.__init__(self, *args)
        super().__init__(**kwargs)


class IODevice(AbstractQObject, QtCore.QIODevice):
    def __init__(self, *args, **kwargs):
        QtCore.QIODevice.__init__(self, *args)
        super().__init__(**kwargs)


class IdentityProxyModel(AbstractQObject, QtCore.QIdentityProxyModel):
    def __init__(self, *args, **kwargs):
        QtCore.QIdentityProxyModel.__init__(self, *args)
        super().__init__(**kwargs)


class ItemSelectionModel(AbstractQObject, QtCore.QItemSelectionModel):
    def __init__(self, *args, **kwargs):
        QtCore.QItemSelectionModel.__init__(self, *args)
        super().__init__(**kwargs)


class Library(AbstractQObject, QtCore.QLibrary):
    def __init__(self, *args, **kwargs):
        QtCore.QLibrary.__init__(self, *args)
        super().__init__(**kwargs)


class MimeData(AbstractQObject, QtCore.QMimeData):
    def __init__(self, *args, **kwargs):
        QtCore.QMimeData.__init__(self, *args)
        super().__init__(**kwargs)


class PauseAnimation(AbstractQObject, QtCore.QPauseAnimation):
    def __init__(self, *args, **kwargs):
        QtCore.QPauseAnimation.__init__(self, *args)
        super().__init__(**kwargs)


class PluginLoader(AbstractQObject, QtCore.QPluginLoader):
    def __init__(self, *args, **kwargs):
        QtCore.QPluginLoader.__init__(self, *args)
        super().__init__(**kwargs)


class Process(AbstractQObject, QtCore.QProcess):
    def __init__(self, *args, **kwargs):
        QtCore.QProcess.__init__(self, *args)
        super().__init__(**kwargs)


class SaveFile(AbstractQObject, QtCore.QSaveFile):
    def __init__(self, *args, **kwargs):
        QtCore.QSaveFile.__init__(self, *args)
        super().__init__(**kwargs)


class SharedMemory(AbstractQObject, QtCore.QSharedMemory):
    def __init__(self, *args, **kwargs):
        QtCore.QSharedMemory.__init__(self, *args)
        super().__init__(**kwargs)


class SignalMapper(AbstractQObject, QtCore.QSignalMapper):
    def __init__(self, *args, **kwargs):
        QtCore.QSignalMapper.__init__(self, *args)
        super().__init__(**kwargs)


class SocketNotifier(AbstractQObject, QtCore.QSocketNotifier):
    def __init__(self, *args, **kwargs):
        QtCore.QSocketNotifier.__init__(self, *args)
        super().__init__(**kwargs)


class SortFilterProxyModel(AbstractQObject, QtCore.QSortFilterProxyModel):
    def __init__(self, *args, **kwargs):
        QtCore.QSortFilterProxyModel.__init__(self, *args)
        super().__init__(**kwargs)


class StringListModel(AbstractQObject, QtCore.QStringListModel):
    def __init__(self, *args, **kwargs):
        QtCore.QStringListModel.__init__(self, *args)
        super().__init__(**kwargs)


class TemporaryFile(AbstractQObject, QtCore.QTemporaryFile):
    def __init__(self, *args, **kwargs):
        QtCore.QTemporaryFile.__init__(self, *args)
        super().__init__(**kwargs)


class Thread(AbstractQObject, QtCore.QThread):
    def __init__(self, *args, **kwargs):
        QtCore.QThread.__init__(self, *args)
        super().__init__(**kwargs)


class ThreadPool(AbstractQObject, QtCore.QThreadPool):
    def __init__(self, *args, **kwargs):
        QtCore.QThreadPool.__init__(self, *args)
        super().__init__(**kwargs)


class TimeLine(AbstractQObject, QtCore.QTimeLine):
    def __init__(self, *args, **kwargs):
        QtCore.QTimeLine.__init__(self, *args)
        super().__init__(**kwargs)


class Timer(AbstractQObject, QtCore.QTimer):
    def __init__(self, *args, **kwargs):
        QtCore.QTimer.__init__(self, *args)
        super().__init__(**kwargs)


class Translator(AbstractQObject, QtCore.QTranslator):
    def __init__(self, *args, **kwargs):
        QtCore.QTranslator.__init__(self, *args)
        super().__init__(**kwargs)


class TransposeProxyModel(AbstractQObject, QtCore.QTransposeProxyModel):
    def __init__(self, *args, **kwargs):
        QtCore.QTransposeProxyModel.__init__(self, *args)
        super().__init__(**kwargs)


class VariantAnimation(AbstractQObject, QtCore.QVariantAnimation):
    def __init__(self, *args, **kwargs):
        QtCore.QVariantAnimation.__init__(self, *args)
        super().__init__(**kwargs)



from typing import Union
from PySide6.QtWidgets import QStackedWidget, QWidget
from ..core.routes import Routes
from ..core import AbstractWidget


class StackedWidget(AbstractWidget, QStackedWidget):
    def __init__(self, 
                 *,
                 children: Union[list[QWidget], Routes, dict] = None,
                 **kwargs):
        QStackedWidget.__init__(self)
        super().__init__(**kwargs)

        if not children:
            return

        if isinstance(children, (Routes, dict)):
            self.routes = Routes("/", **children) if isinstance(children, dict) else children
            self.routes._set_stacked(self)
            for route, child in children.items():
                self.routes.add(route, child)
        else:
            for child in children:
                self.addWidget(child)


#!/usr/bin/python3

from typing import Union
from enum import IntEnum
from PySide6.QtGui import QMovie
from PySide6.QtCore import QByteArray
from PySide6.QtWidgets import QApplication
from ..core import AbstractQObject


class Movie(AbstractQObject, QMovie):
    class Loop(IntEnum):
        infinite: int = -1

    def __init__(self,
                 filename: str = None,
                 *,
                 format: QByteArray | bytes = None,
                 running: bool = False,
                 loop: Union[Loop, int] = Loop.infinite,
                 **kwargs):
        QMovie.__init__(self, QApplication.instance())
        super().__init__(**kwargs)

        # put QApplication.instance() as parent becouse it's global tool.
        # self.setParent(QApplication.instance())

        self.set_loop_count(loop)
        self._current_loop_count = self._get_loop_count(loop)

        self.frameChanged.connect(self._check_loop_count)

        if filename is not None:
            self.setFileName(filename)

        if format is not None:
            self.setFormat(format)

        if running:
            self.start()

    @property
    def current_value(self) -> int:
        return self.currentFrameNumber()

    @property
    def current_loop_count(self) -> int:
        return self._loop_count - self._current_loop_count

    @property
    def is_paused(self) -> bool:
        return self.state() == QMovie.MovieState.Paused

    @property
    def is_running(self) -> bool:
        return self.state() == QMovie.MovieState.Running

    def loopCount(self) -> int:
        return self._loop_count

    def _get_loop_count(self, count) -> int:
        return count.value if isinstance(count, Movie.Loop) else count

    def set_loop_count(self, count: Union[Loop, int]) -> int:
        self._loop_count = self._get_loop_count(count)

    def jump_next(self) -> bool:
        return self.jumpToNextFrame()

    def jump_prev(self) -> bool:
        self.jumpToFrame(self.current_value - 1)

    def pause(self) -> None:
        return self.setPaused(self.state() == QMovie.MovieState.Running)

    def resum(self) -> None:
        return self.setPaused(not (self.state() == QMovie.MovieState.Paused))

    def start(self) -> None:
        return super().start()

    def _check_loop_count(self):
        if self._current_loop_count == -1:
            return

        if self.current_value == 0:
            self._current_loop_count -= 1
            if self._current_loop_count < 0:
                self._current_loop_count = self._loop_count
                self.stop()


# coding: utf-8
from enum import EnumMeta
from typing import Any, Callable


class staticproperty(property):
    '''
    class Object:
        @staticproperty
        def some_method():
            ...
    '''

    def __get__(self, owner_self, owner_cls):         
        return self.fget()


class classproperty(property):
    '''
    class Object:
        
        @classproperty
        def some_method(cls):
            ...
    '''

    def __get__(self, owner_self, owner_cls):
        return self.fget(owner_cls)


class QueryDict(dict):
    '''QueryDict is a dict() that can be queried with dot.

    ::

        d = QueryDict()
        
        # create a key named toto, with the value 1
        d.toto = 1

        # it's the same as
        d['toto'] = 1

    .. versionadded:: 1.0.4
    '''

    def __getattr__(self, attr):
        try:
            return self.__getitem__(attr)
        except KeyError:
            return super(QueryDict, self).__getattr__(attr)

    def __setattr__(self, attr, value):
        self.__setitem__(attr, value)


class EnumDirectValueMeta(EnumMeta):
    '''
    class EnumClass(Enum, metaclass=EnumDirectValueMeta):
        red = 0
        green = 1
        blue = 2

    >> EnumClass.red
    0
    '''
    def __getattribute__(cls, name):
        value = super().__getattribute__(name)
        if isinstance(value, cls):
            value = value.value
        return value


class CheckNone:
    '''
    ### e.g
    NoneCheck(self.setIcon, icon)

    ### same to:
    if icon is not None:
        self.setIcon()
    '''

    def __new__(cls, *args, **kwargs) -> Any:
        instance = super().__new__(cls)
        return instance.__init__(*args, **kwargs)

    def __init__(self, method: Callable, value: Any) -> Any:
        if callable(method) and value is not None:
            return method(value)



#!/usr/bin/python3

from typing import Any, Sequence, Union
from ._declarative import AbstractDec
from ._base import AbstractBase, Func, Args


class AbstractTool:
    '''
    AbstractTool means that the object doesn't contain base methods
    like setObjectName, setProperty ...etc
    '''
    def __init__(self, methods: Union[Sequence[Union[tuple[str, Union[Any, Args]], Func]], 
                                      dict[str, Union[Any, Args]]] = None, **kwargs):
        self._set_methods(methods)
        self._set_property_from_kwargs(**kwargs)

    def _getattr(self, name: str, default: object = None) -> object:
        return AbstractBase._getattr(self, name, default)

    def _set_methods(self, methods) -> None:
        return AbstractBase._set_methods(self, methods)

    def _set_property_from_kwargs(self, **kwargs) -> None:
        for name, value in kwargs.items():
            if hasattr(self, name):
                # handle set callable methods
                if name.startswith("set"):
                    AbstractBase._handle_set_methods(self, name, value)

                # handle add callable methods
                elif name.startswith("add"):
                    AbstractBase._handle_add_methods(self, name, value)


class ToolDec(AbstractTool, AbstractDec):
    pass


# Qtica
__version__ = "0.4.3"
__version_info__ = (0, 4, 3, "", "")

# Python
__minimum_python_version__ = (3, 10)
__maximum_python_version__ = (3, 12)

# PySide6
__pyside_version__ = "6.5.0"


from .core import *


from enum import IntEnum, auto


class Positions(IntEnum):
    left = auto()
    top = auto()
    bottom = auto()
    right = auto()
    center = auto()

    west = left
    north = top
    south = bottom
    east = right


from .caseconverter import CaseConverter, DELIMITERS
from .camel import Camel, camelcase
from .cobol import Cobol, cobolcase
from .flat import Flat, flatcase
from .kebab import Kebab, kebabcase
from .macro import Macro, macrocase
from .pascal import Pascal, pascalcase
from .snake import Snake, snakecase



#!/usr/bin/python3

from typing import Union
from PySide6.QtCore import QSize, Qt
from PySide6.QtWidgets import QWidget
from PySide6.QtGui import (
    QColor, 
    QIcon, 
    QIconEngine, 
    QImage, 
    QMovie, 
    QPaintEvent, 
    QPainter, 
    QPixmap
)
from ..core import AbstractIcons, AbstractWidget
from ..tools.icon import Icon


class IconWidget(AbstractWidget, QWidget):
    def __init__(self,
                icon: Union[str,
                            QIcon,
                            QIconEngine,
                            QPixmap,
                            QImage,
                            AbstractIcons,
                            QMovie],
                *,
                alignment: Qt.AlignmentFlag = Qt.AlignmentFlag.AlignCenter | Qt.AlignmentFlag.AlignHCenter,
                mode: QIcon.Mode = QIcon.Mode.Active,
                state: QIcon.State = QIcon.State.On,
                color: QColor = None,
                size: Union[QSize, tuple, int] = None,
                **kwargs):
        QWidget.__init__(self)
        super().__init__(**kwargs)

        if isinstance(icon, QMovie):
            self._icon = icon
            icon.frameChanged.connect(self.update)
        else:
            self._icon = Icon(icon, color, size)
            self._alignment = alignment
            self._mode = mode
            self._state = state

    @property
    def icon(self) -> QIcon:
        return self._icon

    @icon.setter
    def icon(self, icon) -> None:
        self._icon = Icon(icon)

    def paintEvent(self, event: QPaintEvent) -> None:
        super().paintEvent(event)

        painter = QPainter(self)
        painter.setRenderHints(QPainter.RenderHint.Antialiasing 
                               | QPainter.RenderHint.SmoothPixmapTransform
                               | QPainter.RenderHint.TextAntialiasing)

        if isinstance(self.icon, QMovie):
            painter.drawPixmap(self.rect(),
                               self.icon.currentPixmap())
        else:
            painter.drawPixmap(self.rect(),
                               self._icon.pixmap(self.rect().size(),
                                                 self._mode,
                                                 self._state))

    def setIcon(self, icon) -> None:
        self._icon = Icon(icon)
        self.update()


from enum import Enum


class EnvVars(Enum):
    enable_highdpi_scaling: str = "QT_ENABLE_HIGHDPI_SCALING"
    scale_factor: str = "QT_SCALE_FACTOR"
    quick_controls_style: str = "QT_QUICK_CONTROLS_STYLE"
    quick_controls_fallback_style: str = "QT_QUICK_CONTROLS_FALLBACK_STYLE"
    quick_controls_conf: str = "QT_QUICK_CONTROLS_CONF"
    quick_controls_hover_enabled: str = "QT_QUICK_CONTROLS_HOVER_ENABLED"
    quick_controls_imagine_path: str = "QT_QUICK_CONTROLS_IMAGINE_PATH"
    quick_controls_imagine_smooth: str = "QT_QUICK_CONTROLS_IMAGINE_SMOOTH"
    quick_controls_material_theme: str = "QT_QUICK_CONTROLS_MATERIAL_THEME"
    quick_controls_material_variant: str = "QT_QUICK_CONTROLS_MATERIAL_VARIANT"
    quick_controls_material_accent: str = "QT_QUICK_CONTROLS_MATERIAL_ACCENT"
    quick_controls_material_primary: str = "QT_QUICK_CONTROLS_MATERIAL_PRIMARY"
    quick_controls_material_foreground: str = "QT_QUICK_CONTROLS_MATERIAL_FOREGROUND"
    quick_controls_material_background: str = "QT_QUICK_CONTROLS_MATERIAL_BACKGROUND"
    quick_controls_universal_theme: str = "QT_QUICK_CONTROLS_UNIVERSAL_THEME"
    quick_controls_universal_accent: str = "QT_QUICK_CONTROLS_UNIVERSAL_ACCENT"
    quick_controls_universal_foreground: str = "QT_QUICK_CONTROLS_UNIVERSAL_FOREGROUND"
    quick_controls_universal_background: str = "QT_QUICK_CONTROLS_UNIVERSAL_BACKGROUND"
    android_package_source_dir: str = "QT_ANDROID_PACKAGE_SOURCE_DIR"
    qpa_platform: str = "QT_QPA_PLATFORM"
    im_module: str = "QT_IM_MODULE"
    debug_plugins: str = "QT_DEBUG_PLUGINS"
    x11_no_mitshm: str = "QT_X11_NO_MITSHM"
    version: str = "QT_VERSION"
    packages: str = "QT_PACKAGES"
    select: str = "QT_SELECT"
    style_override: str = "QT_STYLE_OVERRIDE"
    logging_rules: str = "QT_LOGGING_RULES"
    plugin_path: str = "QT_PLUGIN_PATH"
    qpa_platformtheme: str = "QT_QPA_PLATFORMTHEME"


#!/usr/bin/python3

from typing import Union
from PySide6.QtWidgets import QStackedLayout, QStackedWidget, QWidget


class Routes:
    '''
    :Return: dict[str, int]
    '''
    def __init__(self, /, index: str = "/", **kwargs: QWidget) -> None:
        self._index = index
        self._widgets = kwargs

    def _set_stacked(self, stacked: Union[QStackedWidget, QStackedLayout]):
        self._stacked = stacked

    def add(self, route: str, widget: QWidget) -> None:
        self.widgets.__setitem__(route, self._stacked.addWidget(widget))

    def push(self, route: str) -> None:
        self._stacked.setCurrentIndex(self.widgets.__getitem__(route))

    def home(self) -> None:
        self._stacked.setCurrentIndex(self.widgets.__getitem__(self._index))

    def remove(self, route: str) -> None:
        self._stacked.removeWidget(self._stacked.widget(self.widgets.pop(route)))

    def current_route(self) -> str:
        return {v: k for k, v in self.widgets.items()}.get(self._stacked.currentIndex())

    @property
    def stacked(self) -> Union[QStackedWidget, QStackedLayout]:
        return self._stacked
    
    @stacked.setter
    def stacked(self, stacked: Union[QStackedWidget, QStackedLayout]) -> None:
        self._stacked = stacked
    
    @property
    def widgets(self) -> dict:
        return self._widgets


from .clipboard import Clipboard
from .message_handler import MessageHandler
from .launche_url import LauncheURL, UrlOpen
from ._methods import (
    TakeScreenShot,
    showDialog,
    corner_to_edge,
    edge_to_corner,
    center_window,
    mixColor,
    mixLight,
    mixDark,
    randomColor,
    colorToHex,
    parse_css_linear_gradient,
    eliding_text
)


#!/usr/bin/python3

from typing import Any, Callable, Sequence, Union
from PySide6.QtCore import Qt, QObject
from PySide6.QtWidgets import QApplication
from ..enums.events import Events, EventTypeVar
from ..enums.widgets import Widgets, WidgetTypeVar
from ..enums.signals import Signals, SignalTypeVar
from ..utils.caseconverter import camelcase, snakecase


class Func:
    def __init__(self, func, *args, **kwargs):
        self._func = func
        self._args = args
        self._kwargs = kwargs

    def func(self) -> Union[Callable, str]:
        return self._func

    def args(self) -> tuple[Any]:
        return self._args

    def kwargs(self) -> dict[str, Any]:
        return self._kwargs


class Args:
    def __init__(self, *args, **kwargs):
        self._args = args
        self._kwargs = kwargs

    def args(self) -> list[Any]:
        return self._args

    def kwargs(self) -> dict[str, Any]:
        return self._kwargs


class MArgs:
    def __init__(self, *args: Union[Args, Any]):
        self._args = args

    def args(self) -> list[Union[Args, Any]]:
        return self._args


class AbstractBase:
    def __init__(self,
                 uid: str = None,
                 signals: Union[SignalTypeVar, dict[Union[Signals, str], Callable[..., Any]]] = None,
                 events: Union[EventTypeVar, dict[Union[Events, str], Callable[..., Any]]] = None,
                 methods: Union[Sequence[Union[tuple[str, Union[Any, Args]], Func]], dict[str, Union[Any, Args]]] = None,
                 enable_event_stack: bool = True,
                 **kwargs):

        self._enable_event_stack = enable_event_stack

        self._set_uid(uid)
        self._set_events(events)
        self._set_signals(signals)
        self._set_methods(methods)
        self._set_property_from_kwargs(**kwargs)

    def setEventStackEnable(self, state: bool):
        self._enable_event_stack = state

    def _getattr(self, name: str, default: object = None) -> object:
        return getattr(self, name, default)

    def _set_uid(self, uid: str):
        if uid is not None:
            self.setObjectName(uid.strip())

    def _set_methods(self, methods):
        if not methods:
            return

        if isinstance(methods, dict):
            for name, value in methods.items():
                if (func := getattr(self, name)) is not None and callable(func):
                    if isinstance(value, Args):
                        func(*value.args(), **value.kwargs())
                    else:
                        func(value)
        else:
            for method in methods:
                if isinstance(method, Func):
                    if (func := getattr(self, method.func())) is not None and callable(func):
                        func(*method.args(), **method.kwargs())
                elif (func := getattr(self, method[0])) is not None and callable(func):
                    arg = method[1]
                    if isinstance(arg, Args):
                        func(*arg.args(), **arg.kwargs())
                    else:
                        func(arg)

    def _set_events(self, events):
        if not events:
            return

        for event, slot in (events.items() if isinstance(events, dict) else events):
            if isinstance(event, Events):
                _ename = camelcase(event.name) + "Event"
            else:
                _ename = event.strip()

            _ecomp = _ename + "Event"
            if hasattr(self, _ecomp):
                _ename = _ecomp

            self.__setattr__(_ename, slot)

    def _set_signals(self, signals, disconnect: bool = False):
        if not signals:
            return

        for signal, slot in (signals.items() if isinstance(signals, dict) else signals):
            if isinstance(signal, Signals):
                _sname = camelcase(signal.name)
            else:
                _sname = signal.strip()

            if (attr := self._getattr(_sname)) is not None:
                if disconnect:
                    attr.disconnect(slot)
                else:
                    attr.connect(slot)

    def _get_wtype(self, qtype: Union[WidgetTypeVar, QObject] = Widgets.any):
        return qtype.value if isinstance(qtype, Widgets) else qtype

    def fetch(self, uid: str, qtype: Union[WidgetTypeVar, QObject] = Widgets.any) -> QObject:
        return self.findChild(
            self._get_wtype(qtype), 
            uid.strip(), 
            Qt.FindChildOption.FindChildrenRecursively)

    def _set_property_from_kwargs(self, **kwargs):
        for name, value in kwargs.items():
            if hasattr(self, name) or self.property(name) is not None:
                snake_case = snakecase(name)

                # handle signals
                if snake_case in Signals._member_names_:
                    self._getattr(name).connect(value)

                # handle events
                elif snake_case in Events._member_names_:
                    self.__setattr__(name, value)

                # handle parent parametar
                elif name == "parent":
                    self._handle_parent_kwarg(name, value)

                # handle properties
                elif self.property(name) is not None:
                    self.setProperty(name, value)

                # handle set callable methods
                elif name.startswith("set"):
                    self._handle_set_methods(name, value)

                # handle add callable methods
                elif name.startswith("add"):
                    self._handle_add_methods(name, value)

    def _handle_parent_kwarg(self, name: str, value: Any) -> None:
        if isinstance(value, str):
            for parent in QApplication.topLevelWidgets():
                if parent.objectName().strip() == value.strip():
                    self.setParent(parent)

                elif (widget := parent.findChild(QObject, 
                                                 value.strip(), 
                                                 Qt.FindChildOption.FindChildrenRecursively)) is not None:
                    self.setParent(widget)

        elif callable(value):
            self.setParent(value())

        else:
            self.setParent(value)

    def _handle_set_methods(self, name: str, value: Any) -> None:
        if (func := self._getattr(name)) is not None and callable(func):
            if isinstance(value, Args):
                func(*value.args(), **value.kwargs())
            else:
                func(value)

    def _handle_add_methods(self, name: str, value: Any) -> None:
        if (func := self._getattr(name)) is not None and callable(func):
            if isinstance(value, MArgs):
                for v in value.args():
                    if isinstance(v, Args):
                        func(*v.args(), **v.kwargs())
                    else:
                        func(value)
            elif isinstance(value, Args):
                func(*value.args(), **value.kwargs())
            else:
                func(value)

    def __setattr__(self, __name: str, __value: Any) -> None:
        if hasattr(self, "_enable_event_stack") and self._enable_event_stack:
            # Events Stack
            if __name.endswith("Event"):
                __func = self.__getattribute__(__name)
                def __event(e):
                    __func(e)  # load prev events
                    __value(e) # load new event
                    return getattr(self.__class__, __name)

                return super().__setattr__(__name, __event)

        return super().__setattr__(__name, __value)



from enum import Enum
from typing import Union
from PySide6.QtWidgets import QFrame, QWidget, QLayout, QLayoutItem, QStackedLayout
from ..utils.alignment import Alignment
from ..core.routes import Routes
from ..core import AbstractWidget


class Stack(AbstractWidget, QFrame):
    StackingMode: Enum = QStackedLayout.StackingMode
    SizeConstraint: Enum = QStackedLayout.SizeConstraint

    def __init__(self, 
                 *,
                 children: Union[list[Union[QWidget, QLayout, QLayoutItem, Alignment]], Routes] = None,
                 stacking_mode: QStackedLayout.StackingMode = QStackedLayout.StackingMode.StackAll,
                 size_constraint: QStackedLayout.SizeConstraint = None,
                 spacing: int = None,
                 index: Union[int, str] = None,
                 menu_bar: QWidget = None,
                 **kwargs):
        QFrame.__init__(self)

        self.setFrameShape(QFrame.Shape.NoFrame)
        self.setFrameShadow(QFrame.Shadow.Raised)

        super().__init__(**kwargs)

        self._layout = QStackedLayout()

        if menu_bar is not None:
            self._layout.setMenuBar(menu_bar)

        if index is not None and isinstance(index, int):
            self._layout.setCurrentIndex(index)

        if spacing is not None:
            self._layout.setSpacing(spacing)

        if size_constraint is not None:
            self._layout.setSizeConstraint(size_constraint)

        if stacking_mode is not None:
            self._layout.setStackingMode(stacking_mode)

        self._set_children(children)
        self.setLayout(self._layout)

    def _set_children_from_list(self, children):
        for child in children:
            if isinstance(child, Alignment):
                _widget = child.child

                if isinstance(_widget, QWidget):
                    _func = self._layout.addWidget
                elif isinstance(_widget, QLayoutItem):
                    _func = self._layout.addItem
                elif isinstance(_widget, QLayout):
                    _widget_wrapper = QWidget(self)
                    _widget.setProperty("parent", _widget_wrapper)
                    _widget_wrapper.setLayout(_widget)
                    _func = self._layout.addWidget

                _func(_widget)
                self.setAlignment(_widget, child.alignment)

            elif isinstance(child, QLayoutItem):
                self._layout.addItem(child)

            elif isinstance(child, QWidget):
                self._layout.addWidget(child)

            elif isinstance(child, QLayout):
                _widget = QWidget(self)
                child.setProperty("parent", _widget)
                _widget.setLayout(child)
                self._layout.addWidget(_widget)

    def _set_children_from_dict(self, children):
        self.routes = Routes(**children.items()) if isinstance(children, dict) else children
        self.routes._set_stacked(self)

        for route, child in self.routes.widgets:
            if isinstance(child, Alignment):
                _widget = child.child

                if isinstance(_widget, QLayout):
                    _widget = QWidget(self)
                    _widget.setProperty("parent", _widget)
                    _widget.setLayout(_widget)
                    self.routes.add(route, _widget)
                    self.setAlignment(_widget, child.alignment)

                elif isinstance(_widget, QWidget):
                    self.routes.add(route, _widget)
                    self.setAlignment(_widget, child.alignment)
            else:
                self.routes.add(route, child)

    def _set_children(self, children):
        if not children:
            return

        if isinstance(children, (Routes, dict)):
            return self._set_children_from_dict(children)
        return self._set_children_from_list(children)

    @property
    def stacked(self) -> QStackedLayout:
        return self._layout



#!/usr/bin/python3

from PySide6 import QtCore, QtGui
from ...core import AbstractTool


class BackingStore(AbstractTool, QtGui.QBackingStore):
    def __init__(self, *args, **kwargs):
        QtGui.QBackingStore.__init__(self, *args)
        super().__init__(**kwargs)


class Bitmap(AbstractTool, QtGui.QBitmap):
    def __init__(self, *args, **kwargs):
        QtGui.QBitmap.__init__(self, *args)
        super().__init__(**kwargs)


class Brush(AbstractTool, QtGui.QBrush):
    def __init__(self, *args, **kwargs):
        QtGui.QBrush.__init__(self, *args)
        super().__init__(**kwargs)


class ColorSpace(AbstractTool, QtGui.QColorSpace):
    def __init__(self, *args, **kwargs):
        QtGui.QColorSpace.__init__(self, *args)
        super().__init__(**kwargs)


class ConicalGradient(AbstractTool, QtGui.QConicalGradient):
    def __init__(self, *args, **kwargs):
        QtGui.QConicalGradient.__init__(self, *args)
        super().__init__(**kwargs)


class Cursor(AbstractTool, QtGui.QCursor):
    def __init__(self, *args, **kwargs):
        QtGui.QCursor.__init__(self, *args)
        super().__init__(**kwargs)


class DesktopServices(AbstractTool, QtGui.QDesktopServices):
    def __init__(self, *args, **kwargs):
        QtGui.QDesktopServices.__init__(self, *args)
        super().__init__(**kwargs)


class Font(AbstractTool, QtGui.QFont):
    def __init__(self, *args, **kwargs):
        QtGui.QFont.__init__(self, *args)
        super().__init__(**kwargs)


class GlyphRun(AbstractTool, QtGui.QGlyphRun):
    def __init__(self, *args, **kwargs):
        QtGui.QGlyphRun.__init__(self, *args)
        super().__init__(**kwargs)


class Gradient(AbstractTool, QtGui.QGradient):
    def __init__(self, *args, **kwargs):
        QtGui.QGradient.__init__(self, *args)
        super().__init__(**kwargs)


class Image(AbstractTool, QtGui.QImage):
    def __init__(self, *args, **kwargs):
        QtGui.QImage.__init__(self, *args)
        super().__init__(**kwargs)


class ImageIOHandler(AbstractTool, QtGui.QImageIOHandler):
    def __init__(self, *args, **kwargs):
        QtGui.QImageIOHandler.__init__(self, *args)
        super().__init__(**kwargs)


class ImageReader(AbstractTool, QtGui.QImageReader):
    def __init__(self, *args, **kwargs):
        QtGui.QImageReader.__init__(self, *args)
        super().__init__(**kwargs)


class ImageWriter(AbstractTool, QtGui.QImageWriter):
    def __init__(self, *args, **kwargs):
        QtGui.QImageWriter.__init__(self, *args)
        super().__init__(**kwargs)


class LinearGradient(AbstractTool, QtGui.QLinearGradient):
    def __init__(self, *args, **kwargs):
        QtGui.QLinearGradient.__init__(self, *args)
        super().__init__(**kwargs)


class PageLayout(AbstractTool, QtGui.QPageLayout):
    def __init__(self, *args, **kwargs):
        QtGui.QPageLayout.__init__(self, *args)
        super().__init__(**kwargs)


class PagedPaintDevice(AbstractTool, QtGui.QPagedPaintDevice):
    def __init__(self, *args, **kwargs):
        QtGui.QPagedPaintDevice.__init__(self, *args)
        super().__init__(**kwargs)


class PaintEngine(AbstractTool, QtGui.QPaintEngine):
    def __init__(self, *args, **kwargs):
        QtGui.QPaintEngine.__init__(self, *args)
        super().__init__(**kwargs)


class Painter(AbstractTool, QtGui.QPainter):
    def __init__(self, *args, **kwargs):
        QtGui.QPainter.__init__(self, *args)
        super().__init__(**kwargs)


class PainterPathStroker(AbstractTool, QtGui.QPainterPathStroker):
    def __init__(self, *args, **kwargs):
        QtGui.QPainterPathStroker.__init__(self, *args)
        super().__init__(**kwargs)


class Palette(AbstractTool, QtGui.QPalette):
    def __init__(self, *args, **kwargs):
        QtGui.QPalette.__init__(self, *args)
        super().__init__(**kwargs)


class Pen(AbstractTool, QtGui.QPen):
    def __init__(self, *args, **kwargs):
        QtGui.QPen.__init__(self, *args)
        super().__init__(**kwargs)


class Picture(AbstractTool, QtGui.QPicture):
    def __init__(self, *args, **kwargs):
        QtGui.QPicture.__init__(self, *args)
        super().__init__(**kwargs)


class Pixmap(AbstractTool, QtGui.QPixmap):
    def __init__(self, *args, **kwargs):
        QtGui.QPixmap.__init__(self, *args)
        super().__init__(**kwargs)


class Quaternion(AbstractTool, QtGui.QQuaternion):
    def __init__(self, *args, **kwargs):
        QtGui.QQuaternion.__init__(self, *args)
        super().__init__(**kwargs)


class RadialGradient(AbstractTool, QtGui.QRadialGradient):
    def __init__(self, *args, **kwargs):
        QtGui.QRadialGradient.__init__(self, *args)
        super().__init__(**kwargs)


class Rgba64(AbstractTool, QtGui.QRgba64):
    def __init__(self, *args, **kwargs):
        QtGui.QRgba64.__init__(self, *args)
        super().__init__(**kwargs)

if QtCore.QSysInfo.buildCpuArchitecture() != "arm64":
    class RhiBuffer(AbstractTool, QtGui.QRhiBuffer):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiBuffer.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiColorAttachment(AbstractTool, QtGui.QRhiColorAttachment):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiColorAttachment.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiCommandBuffer(AbstractTool, QtGui.QRhiCommandBuffer):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiCommandBuffer.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiComputePipeline(AbstractTool, QtGui.QRhiComputePipeline):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiComputePipeline.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiDepthStencilClearValue(AbstractTool, QtGui.QRhiDepthStencilClearValue):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiDepthStencilClearValue.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiGraphicsPipeline(AbstractTool, QtGui.QRhiGraphicsPipeline):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiGraphicsPipeline.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiReadbackDescription(AbstractTool, QtGui.QRhiReadbackDescription):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiReadbackDescription.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiRenderBuffer(AbstractTool, QtGui.QRhiRenderBuffer):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiRenderBuffer.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiRenderTarget(AbstractTool, QtGui.QRhiRenderTarget):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiRenderTarget.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiSampler(AbstractTool, QtGui.QRhiSampler):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiSampler.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiScissor(AbstractTool, QtGui.QRhiScissor):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiScissor.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiShaderResourceBindings(AbstractTool, QtGui.QRhiShaderResourceBindings):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiShaderResourceBindings.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiShaderStage(AbstractTool, QtGui.QRhiShaderStage):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiShaderStage.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiSwapChain(AbstractTool, QtGui.QRhiSwapChain):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiSwapChain.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiSwapChainRenderTarget(AbstractTool, QtGui.QRhiSwapChainRenderTarget):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiSwapChainRenderTarget.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiTexture(AbstractTool, QtGui.QRhiTexture):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiTexture.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiTextureCopyDescription(AbstractTool, QtGui.QRhiTextureCopyDescription):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiTextureCopyDescription.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiTextureRenderTarget(AbstractTool, QtGui.QRhiTextureRenderTarget):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiTextureRenderTarget.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiTextureRenderTargetDescription(AbstractTool, QtGui.QRhiTextureRenderTargetDescription):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiTextureRenderTargetDescription.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiTextureSubresourceUploadDescription(AbstractTool, QtGui.QRhiTextureSubresourceUploadDescription):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiTextureSubresourceUploadDescription.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiTextureUploadDescription(AbstractTool, QtGui.QRhiTextureUploadDescription):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiTextureUploadDescription.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiTextureUploadEntry(AbstractTool, QtGui.QRhiTextureUploadEntry):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiTextureUploadEntry.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiVertexInputAttribute(AbstractTool, QtGui.QRhiVertexInputAttribute):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiVertexInputAttribute.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiVertexInputBinding(AbstractTool, QtGui.QRhiVertexInputBinding):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiVertexInputBinding.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiVertexInputLayout(AbstractTool, QtGui.QRhiVertexInputLayout):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiVertexInputLayout.__init__(self, *args)
            super().__init__(**kwargs)


    class RhiViewport(AbstractTool, QtGui.QRhiViewport):
        def __init__(self, *args, **kwargs):
            QtGui.QRhiViewport.__init__(self, *args)
            super().__init__(**kwargs)


    class Shader(AbstractTool, QtGui.QShader):
        def __init__(self, *args, **kwargs):
            QtGui.QShader.__init__(self, *args)
            super().__init__(**kwargs)


    class ShaderCode(AbstractTool, QtGui.QShaderCode):
        def __init__(self, *args, **kwargs):
            QtGui.QShaderCode.__init__(self, *args)
            super().__init__(**kwargs)


    class ShaderKey(AbstractTool, QtGui.QShaderKey):
        def __init__(self, *args, **kwargs):
            QtGui.QShaderKey.__init__(self, *args)
            super().__init__(**kwargs)


    class ShaderVersion(AbstractTool, QtGui.QShaderVersion):
        def __init__(self, *args, **kwargs):
            QtGui.QShaderVersion.__init__(self, *args)
            super().__init__(**kwargs)


class StandardItem(AbstractTool, QtGui.QStandardItem):
    def __init__(self, *args, **kwargs):
        QtGui.QStandardItem.__init__(self, *args)
        super().__init__(**kwargs)


class StaticText(AbstractTool, QtGui.QStaticText):
    def __init__(self, *args, **kwargs):
        QtGui.QStaticText.__init__(self, *args)
        super().__init__(**kwargs)


class SurfaceFormat(AbstractTool, QtGui.QSurfaceFormat):
    def __init__(self, *args, **kwargs):
        QtGui.QSurfaceFormat.__init__(self, *args)
        super().__init__(**kwargs)


class TextBlock(AbstractTool, QtGui.QTextBlock):
    def __init__(self, *args, **kwargs):
        QtGui.QTextBlock.__init__(self, *args)
        super().__init__(**kwargs)


class TextBlockFormat(AbstractTool, QtGui.QTextBlockFormat):
    def __init__(self, *args, **kwargs):
        QtGui.QTextBlockFormat.__init__(self, *args)
        super().__init__(**kwargs)


class TextCharFormat(AbstractTool, QtGui.QTextCharFormat):
    def __init__(self, *args, **kwargs):
        QtGui.QTextCharFormat.__init__(self, *args)
        super().__init__(**kwargs)


class TextCursor(AbstractTool, QtGui.QTextCursor):
    def __init__(self, *args, **kwargs):
        QtGui.QTextCursor.__init__(self, *args)
        super().__init__(**kwargs)


class TextDocumentWriter(AbstractTool, QtGui.QTextDocumentWriter):
    def __init__(self, *args, **kwargs):
        QtGui.QTextDocumentWriter.__init__(self, *args)
        super().__init__(**kwargs)


class TextFormat(AbstractTool, QtGui.QTextFormat):
    def __init__(self, *args, **kwargs):
        QtGui.QTextFormat.__init__(self, *args)
        super().__init__(**kwargs)


class TextFrameFormat(AbstractTool, QtGui.QTextFrameFormat):
    def __init__(self, *args, **kwargs):
        QtGui.QTextFrameFormat.__init__(self, *args)
        super().__init__(**kwargs)


class TextImageFormat(AbstractTool, QtGui.QTextImageFormat):
    def __init__(self, *args, **kwargs):
        QtGui.QTextImageFormat.__init__(self, *args)
        super().__init__(**kwargs)


class TextInlineObject(AbstractTool, QtGui.QTextInlineObject):
    def __init__(self, *args, **kwargs):
        QtGui.QTextInlineObject.__init__(self, *args)
        super().__init__(**kwargs)


class TextLayout(AbstractTool, QtGui.QTextLayout):
    def __init__(self, *args, **kwargs):
        QtGui.QTextLayout.__init__(self, *args)
        super().__init__(**kwargs)


class TextLine(AbstractTool, QtGui.QTextLine):
    def __init__(self, *args, **kwargs):
        QtGui.QTextLine.__init__(self, *args)
        super().__init__(**kwargs)


class TextListFormat(AbstractTool, QtGui.QTextListFormat):
    def __init__(self, *args, **kwargs):
        QtGui.QTextListFormat.__init__(self, *args)
        super().__init__(**kwargs)


class TextOption(AbstractTool, QtGui.QTextOption):
    def __init__(self, *args, **kwargs):
        QtGui.QTextOption.__init__(self, *args)
        super().__init__(**kwargs)


class TextTableCell(AbstractTool, QtGui.QTextTableCell):
    def __init__(self, *args, **kwargs):
        QtGui.QTextTableCell.__init__(self, *args)
        super().__init__(**kwargs)


class TextTableCellFormat(AbstractTool, QtGui.QTextTableCellFormat):
    def __init__(self, *args, **kwargs):
        QtGui.QTextTableCellFormat.__init__(self, *args)
        super().__init__(**kwargs)


class TextTableFormat(AbstractTool, QtGui.QTextTableFormat):
    def __init__(self, *args, **kwargs):
        QtGui.QTextTableFormat.__init__(self, *args)
        super().__init__(**kwargs)


class UndoCommand(AbstractTool, QtGui.QUndoCommand):
    def __init__(self, *args, **kwargs):
        QtGui.QUndoCommand.__init__(self, *args)
        super().__init__(**kwargs)


class Vector2D(AbstractTool, QtGui.QVector2D):
    def __init__(self, *args, **kwargs):
        QtGui.QVector2D.__init__(self, *args)
        super().__init__(**kwargs)


class Vector3D(AbstractTool, QtGui.QVector3D):
    def __init__(self, *args, **kwargs):
        QtGui.QVector3D.__init__(self, *args)
        super().__init__(**kwargs)


class Vector4D(AbstractTool, QtGui.QVector4D):
    def __init__(self, *args, **kwargs):
        QtGui.QVector4D.__init__(self, *args)
        super().__init__(**kwargs)



from typing import Any, Union
from PySide6.QtCore import Qt, QObject
from PySide6.QtWidgets import QApplication, QWidget
from ..enums.widgets import Widgets, WidgetTypeVar
from ._declarative import AbstractDec, TrackingDec


class Api:
    '''

    ```
    api = Api(parent)
    obj = api.get('uid')
    >> obj = object
    ```

    ### Direct Access
    ```
    Api.fetch('uid')
    >> object
    ```
    '''
    def __init__(self, 
                 parent: QObject = None,
                 qtype: WidgetTypeVar | object = Widgets.any):

        self.parent = parent
        self.qtype = qtype

    def _find_child(self, 
                    target: QObject,
                    uid: str,
                    qtype: WidgetTypeVar) -> QObject | None:

        return target.findChild(self._get_wtype(qtype 
                                                if qtype is not None
                                                else self.qtype),
                                uid.strip(), 
                                Qt.FindChildOption.FindChildrenRecursively)

    def _get_wtype(self, qtype: WidgetTypeVar = Widgets.any):
        return (qtype.value 
                if isinstance(qtype, Widgets) 
                else qtype)

    def get(self,
            uid: str, 
            qtype: WidgetTypeVar | object = Widgets.any) -> QObject | None:

        uid = uid.strip()

        if Api.isinstance(qtype, AbstractDec):
            return TrackingDec.get(uid)

        if self.parent is not None:
            return self._find_child(self.parent, uid, qtype)

        for parent in QApplication.topLevelWidgets():
            widget = self._find_child(parent, uid, qtype)
            if widget is not None:
                return widget

    @staticmethod
    def isinstance(obj, _class) -> bool:
        if obj == _class:
            return True

        if type(obj) == _class:
            return True

        if isinstance(obj, _class):
            return True

        return False

    @classmethod
    def fetch(cls,
              uid: str,
              qtype: Any = Widgets.any
        ) -> Union[QWidget, QObject, Any, None]:

        uid = uid.strip()

        if uid == QApplication.instance().objectName().strip():
            return QApplication.instance()

        if cls.isinstance(qtype, AbstractDec):
            return TrackingDec.get(uid)

        if uid in TrackingDec._instances_dict:
            return TrackingDec.get(uid)

        qobjs_copy = set(QApplication.allWidgets().copy())
        qobjs_copy.update(QApplication.allWindows())
        qobjs_copy.update(QApplication.topLevelWidgets())
        qobjs_copy.update(QApplication.topLevelWindows())
        qobjs_copy.update(QApplication.screens())
        for parent in qobjs_copy:
            if uid == parent.objectName().strip():
                return parent
            if (widget := cls._find_child(cls, parent, uid, qtype)) is not None:
                return widget
        del qobjs_copy

        return cls._find_child(cls, QApplication.instance(), uid, qtype)

    @staticmethod
    def dec_fetch(uid: str) -> Any:
        return TrackingDec.get(uid)

    @staticmethod
    def declarative_fetch(uid: str) -> Any:
        return Api.dec_fetch(uid)

    def __call__(self, *args: Any, **kwds: Any) -> Any:
        return self.get(*args, **kwds)